import java.util.*;                       
import java.io.*;
import static java.lang.Math.*;
import java.math.*;

public class Gen implements Runnable {
	BufferedReader in;
	//PrintWriter out;
	PrintStream out, outA = System.out;  
	StringTokenizer st = new StringTokenizer("");
	
	static boolean random = true;
	static double eps = 1e-7;
	static Random rand;
	char [][] b;
	static int n, m, k, testNumber = -1; // высота и ширина картинки и количество вершин многоугольника
	final int mLen = 50; // минимальная длина стороны
	final String [] names = new String [] {"TRIANGLE", "SQUARE", "PENTAGON"};
	                  
	// parameters: 1) "test" testNumber 2) testNumber width height vertices
	public static void main(String [] args) throws Exception {
		if (args.length > 0) {
			if (args[0].equals("test")) {
				testNumber = new Integer(args[1]);
				m = n = k = -1;
			} else {
      			random = false;
      			testNumber = new Integer(args[0]);
      			m = new Integer(args[1]);
      			n = new Integer(args[2]);
      			k = new Integer(args[3]);
			}
			rand = new Random(testNumber);
		}
		new Thread(new Gen()).start();
	}

	// вывод картинки
	void printTest() {
		out.println(n + " " + m);
		for (int i = 0; i < n; i++) {
			for (int j = 0; j < m; j++) {
				out.print(b[i][j]);
			}
			out.println();
		}
	}

	// заполнение всех квадратиков, пересекаемых отрезком p1<--->p2
	void put(P p1, P p2) {
		double minX = min(p1.x, p2.x), maxX = max(p1.x, p2.x), minY = min(p1.y, p2.y), maxY = max(p1.y, p2.y);
		for (int i = 0; i < n; i++) {
			for (int j = 0; j < m; j++) {
				for (int e = 0; e < 2; e++) {
					double y = p1.yCross(p2, j + e);
					if (y > i - eps && y < i + 1 + eps && y > minY - eps && y < maxY + eps) b[i][j] = '#';
					double x = p1.xCross(p2, i + e);
					if (x > j - eps && x < j + 1 + eps && x > minX - eps && x < maxX + eps) b[i][j] = '#';
				}
			}		
		}
	}

	public void run() {
		try {
			Locale.setDefault(Locale.US);
			if (testNumber != -1) {
				String zero = testNumber < 10 ? "0" : "";
				String fileName = "../tests/" + zero + testNumber;
				out = new PrintStream(fileName);
				outA = new PrintStream(fileName + ".a");
			} else {
				out = new PrintStream("input.txt");
			}
			if (random) {
				m = rand.nextInt(101) + 150;
				n = rand.nextInt(101) + 150;
				k = rand.nextInt(3) + 3;
			}
			assert (150 <= n && n <= 250);
			assert (150 <= m && m <= 250);
			assert (3 <= k && k <= 5);
			b = new char[n][m];
			for (int i = 0; i < n; i++) {
				Arrays.fill(b[i], '.');
			}

			outA.println(names[k - 3]);
			// fi --- случайный угол поворота вокруг центра
			// psi --- угол между смежными векторами из центра
			double fi = rDouble(0, 2 * PI), psi = 2 * PI / k;

			// minR --- минимальный радиус, такой, что сторона >= mLen
			// если центр O многоугольника лежит в прямоугольнике (R, R)<-->(m-R, n-R),
			// то он гарантированно влезает в прямоугольник
			double minR = mLen / sqrt(2 - 2 * cos(psi)), maxR = min(m, n) / 2D;
			// R --- радиус описанной окружности
			double R = rDouble(minR, maxR);

			P [] vertices = new P[k];
			P O = rPoint(R + 2, R + 2, m - R - 2, n - R - 2);
			P r_vector = new P(R, 0).rotate(fi);
			for (int i = 0; i < k; i++) {
				vertices[i] = O.add(r_vector);
				r_vector = r_vector.rotate(psi);
			}
			for (int i = 0; i < k; i++) {
				put(vertices[i], vertices[(i + 1) % k]);
			}

			printTest();
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	double rDouble(double l, double r) {
		double rVal = rand.nextDouble();
		return l + rVal * (r - l);
	}

	P rPoint(double minX, double minY, double maxX, double maxY) {
		return new P(rDouble(minX, maxX), rDouble(minY, maxY));
	}
}

class P {
	double x, y;
	P() {}
	P(double x, double y) { this.x = x; this.y = y; }
	P add(P r) { return new P(x + r.x, y + r.y); }
	P subtract(P r) { return new P(x - r.x, y - r.y); }
	P rotate(double fi) { return new P(x * cos(fi) - y * sin(fi), x * sin(fi) + y * cos(fi)); }
	P mult(double c) { return new P(x * c, y * c); }
	double length() { return hypot(x, y); }
	P norm() { return this.mult(1D / length()); }
	double det(P r) { return x * r.y - y * r.x; }
	double dot(P r) { return x * r.x + y * r.y; }
	double dist(P l1, P l2) { return l2.subtract(this).det(l1.subtract(this)) / l2.subtract(l1).length(); }
	// ордината пересечения прямой this<-->r и прямой x = xc
	double yCross(P r, double xc) { return (new P(x - xc, y).det(new P(r.x - xc, r.y))) / (x - r.x); }
	double xCross(P r, double yc) { return (new P(x, y - yc).det(new P(r.x, r.y - yc))) / (r.y - y); }
}
