#!/usr/bin/env python2

from amtest import *
from bz2 import BZ2File
import pickle

MAX_N = 200

def queens(n):
  a = [None] * n
  o = n % 6
  m = n>>1
  if o in [0,4]:
    for i in range(m):
      a[i] = 1 + 2 * i
      a[m+i] = 2 * i
  elif o == 2:
    for i in range(1,m-2):
      a[i] = 2 * i
    for i in range(m+2,n-1):
      a[i] = 5 + 2 * (i - m - 2)
    a[0] = n-4
    a[m-2] = 0
    a[m-1] = n-2
    a[m] = 1
    a[m+1] = n-1
    a[n-1] = 3
  else:
    a = queens(n-1)
    a.append(n-1)
  for i in range(n):
    for j in range(i+1,n):
      assert(not (abs(a[j]-a[i]) in [0, j-i]))
  return a

def cvt(z):
  M = len(z)
  b = list(map(len, z))
  N = min(b)
  assert(N == max(b))
  a = Matrix.make(M,N)
  for j in range(M):
    for i in range(N):
      a[j][i] = ord(z[j][i]) - 48
  return M,N,a

def pt(z,desc=''):
  M = len(z)
  b = list(map(len, z))
  N = min(b)
  desc += ' M = %d, N = %d' % (M,N)
  assert(N == max(b))
  f.begin_test(desc)
  f.write('%d %d\n' % (M,N))
  for s in z: f.write(s + '\n')
  f.end_test()

def pt2(m,n,a,desc=''):
  z = []
  for j in range(m):
    s = ''
    for i in range(n):
      s += chr(48 + a[j][i])
    z.append(s)
  pt(z,desc)

def pt2_rotate(m,n,a):
  z = []
  for i in range(n):
    s = ''
    for j in range(m):
      s += chr(48 + ((6 + a[j][i]) & 7))
    z.append(s)
  pt(z)

def rt(m,n):
  a = Matrix.make(m,n)
  for j in range(m):
    for i in range(n):
      a[j][i] = random.randrange(8)
  pt2(m,n,a,'rt()')

def rt2(m,n):
  a = Matrix.make(m,n)
  for i in range(m):
    for j in range(n):
      a[j][i] = random.randrange(8)
  pt2(m,n,a,'rt2()')


def sample():
  pt(['0000','4444','0000'])
  pt(['06','24'])
  pt(['4','4'])

def pt_2tests(z):
  pt(z)
  M,N,a = cvt(z)
  pt2_rotate(M,N,a)

def sgn(x):
  if x<0:return -1
  if x>0:return  1
  return 0

def rt_path(m,n,v,sigma):
  a = Matrix.make(m,n,default=0)
  x = n>>1
  y = m>>1
  o = 0
  for t in range(4*m*n):
    u = (o + int(round(random.gauss(0,sigma)))) & 7
    i = x + v[u][0]
    j = y + v[u][1]
    if (i>=0)and(i<n)and(j>=0)and(j<m):
      a[y][x] = u
      x,y = i,j
  pt2(m,n,a,'rt_path()')

def small():
  pt(['7'])
  pt(['024'])
  pt_2tests(['40'])
  pt_2tests(['04' * 3])
  pt_2tests(['4' + '04' * 3])
  pt_2tests(['1' * 10])
  rt(1,10)
  rt(10,1)
  rt(5,4)
  rt(4,5)
  r = [5,10,15]
  for m,n in sorted([(x,y) for x in r for y in r],key = lambda x:x[0]*x[1]):
    rt(m,n)

def rt3(seed, changed):
  random.seed(seed)
  m = n = MAX_N
  a = Matrix.make(m,n)
  for j in range(m):
    for i in range(n):
      a[j][i] = random.randrange(8)
  for j,i,c in changed: a[j][i] = c
  pt2(m,n,a,'rt3()')

def single_soldier_alive_in_row(n,pos):
  assert((n & 1) == 0)
  a = [None] * n
  if (pos & 1) == 0:
    for i in range(0,pos,2):
      a[i] = 0
      a[i+1] = 4
    for i in range(pos,n): a[i] = 0
  else:
    for i in range(n-1,pos,-2):
      a[i] = 4
      a[i-1] = 0
    for i in range(pos+1): a[i] = 4
  return a

def maxans(n):
  assert((n & 1) == 0)
  x = queens(n)
  a = []
  for i in range(n):
    a.append(single_soldier_alive_in_row(n, x[i]))
  pt2(n,n,a)

def bz2_test(filename):
  g = BZ2File(filename,'rb')
  a = pickle.load(g)
  g.close()
  pt(a,filename)

def big():
  pt(['024' * (MAX_N // 3)] * MAX_N)
  for t in  range(10): rt(MAX_N,MAX_N)
  for c in ['0','1']:
    pt(MAX_N * [c * MAX_N])
  a = Matrix.make(MAX_N,MAX_N)
  for j in range(MAX_N):
    for i in range(MAX_N):
      a[j][i] = (i+j)&7
  pt2(MAX_N,MAX_N,a)
  k = MAX_N>>1
  for j in range(k): a[j] = single_soldier_alive_in_row(MAX_N, 2 * j)
  pt2(k,MAX_N,a)
  v = [(1,0),(1,-1),(0,-1),(-1,-1),(-1,0),(-1,1),(0,1),(1,1)]

  d = {}
  d[(0,0)] = 0
  for i in range(8): d[v[i]] = i
  for j in range(MAX_N):
    dy = (MAX_N >> 1) - j
    for i in range(MAX_N):
      dx = (MAX_N >> 1) - i
      a[j][i] = d.get( (sgn(dx),sgn(dy)), None)
  pt2(MAX_N, MAX_N, a)

  for sigma in [0.5,1.0,2.0,4.0,8.0]:
    rt_path(MAX_N,MAX_N,v,sigma)

  random.seed(144)
  rt2(MAX_N, MAX_N)

  random.seed(1867)
  rt(MAX_N, MAX_N)

  random.seed(631)
  rt(1, MAX_N)

  #138 soldiers alive
  rt3(4,[(16, 32, 3), (36, 155, 5), (157, 120, 3), (133, 45, 2), (4, 32, 5), (90, 163, 2), (65, 66, 3), (154, 102, 5), (18, 85, 7), (150, 176, 3), (26, 113, 2), (154, 181, 1), (75, 99, 6), (180, 137, 7), (117, 115, 6), (107, 28, 6), (99, 57, 7), (130, 100, 1), (165, 14, 4), (98, 7, 7), (120, 55, 7), (198, 107, 1), (120, 166, 7), (25, 167, 6), (29, 154, 3), (191, 120, 4), (13, 20, 4), (117, 189, 2), (167, 190, 0), (75, 156, 0), (57, 102, 1), (141, 40, 1), (22, 70, 7), (101, 69, 7), (87, 18, 1), (76, 161, 3), (85, 34, 5), (111, 138, 2), (81, 12, 0), (46, 65, 7), (37, 196, 2), (6, 54, 3), (196, 112, 3), (181, 48, 5), (189, 104, 2), (147, 32, 2), (64, 195, 2), (197, 88, 7), (128, 67, 0), (73, 150, 6), (170, 29, 7), (2, 83, 7), (91, 63, 1), (80, 64, 7), (79, 193, 4), (150, 196, 0), (6, 134, 7), (111, 123, 7)])
  #139 soldiers alive
  rt3(1,[(1, 112, 7), (6, 119, 6), (9, 166, 7), (12, 195, 2), (16, 67, 3), (17, 190, 5), (18, 90, 7), (18, 125, 7), (21, 81, 5), (28, 73, 4), (36, 49, 1), (36, 198, 4), (38, 159, 6), (48, 8, 7), (51, 36, 5), (53, 49, 2), (57, 8, 5), (67, 189, 1), (68, 85, 6), (72, 171, 7), (78, 160, 1), (80, 7, 4), (84, 20, 6), (85, 9, 5), (91, 13, 7), (98, 198, 7), (99, 109, 3), (99, 179, 6), (100, 39, 5), (104, 15, 1), (106, 15, 7), (108, 155, 7), (108, 172, 1), (110, 4, 0), (113, 37, 4), (123, 136, 1), (130, 103, 5), (130, 142, 0), (133, 45, 5), (135, 10, 5), (147, 7, 3), (155, 40, 7), (155, 131, 1), (165, 67, 0), (166, 148, 1), (171, 9, 0), (180, 59, 7), (181, 192, 1), (182, 166, 6), (185, 139, 4), (186, 98, 2), (189, 1, 0), (189, 133, 4), (189, 142, 7), (190, 179, 7), (195, 2, 0), (196, 101, 4), (197, 22, 0)])
  #146 soldiers  alive
  rt3(144,[(3, 144, 3), (4, 186, 3), (5, 73, 3), (6, 119, 3), (8, 162, 0), (10, 137, 6), (11, 20, 2), (14, 124, 0), (14, 136, 1), (20, 67, 2), (20, 143, 7), (21, 116, 4), (22, 69, 5), (23, 92, 0), (24, 24, 2), (34, 100, 2), (39, 88, 7), (40, 190, 4), (44, 25, 4), (46, 66, 3), (46, 68, 5), (51, 68, 5), (53, 59, 7), (56, 12, 7), (61, 5, 4), (62, 160, 3), (66, 156, 2), (70, 72, 1), (71, 2, 5), (71, 80, 4), (73, 7, 7), (74, 11, 6), (74, 96, 2), (77, 158, 2), (78, 180, 0), (81, 119, 2), (82, 199, 6), (84, 166, 7), (86, 26, 5), (88, 150, 5), (89, 82, 0), (90, 174, 3), (100, 89, 1), (103, 5, 1), (103, 22, 0), (108, 172, 0), (108, 187, 7), (109, 39, 3), (110, 154, 4), (113, 5, 5), (114, 30, 0), (116, 181, 4), (120, 124, 6), (126, 70, 4), (135, 114, 1), (136, 3, 5), (138, 119, 2), (146, 165, 5), (154, 176, 5), (156, 114, 5), (156, 148, 3), (157, 138, 1), (158, 25, 5), (160, 148, 6), (167, 28, 5), (167, 172, 0), (170, 130, 3), (171, 6, 4), (175, 52, 3), (180, 58, 4), (183, 92, 2), (184, 165, 6), (185, 22, 7), (187, 79, 4), (191, 135, 4), (196, 43, 5), (196, 147, 3), (197, 174, 6), (199, 132, 3)])

  bz2_test('ans172.bz2')
  bz2_test('maxshots.bz2')
  bz2_test('maxtime.bz2')
  bz2_test('ans189.bz2')
  for n in range(MAX_N-6,MAX_N+1,2):
    maxans(n)

def main():
  global f
  f = TestGen(1,problem='soldiers', tests_path=os.path.join('..','tests'))    
  sample()
  small()
  big()

  f.close()

if __name__ == '__main__': main()
