import java.util.*;
import java.io.*;

public class timetable_sb {

	final static String FNAME = "timetable";

	public Parser parser;
	public PrintWriter out;
	
	void open() throws IOException {
		Locale.setDefault( Locale.US );
		parser = new Parser( new BufferedReader( new FileReader( new File( FNAME + ".in" ) ) ) );
		out = new PrintWriter( new File( FNAME + ".out" ) );
	}

	void close() throws IOException {
		out.close();
	}

	class Lecture implements Comparable < Lecture > {
		int s;
		int e;

		Lecture( int S, int E ) {
			s = S;
			e = E;
		}

		public int compareTo( Lecture l ) {
			if ( s != l.s ) {
				return new Integer( s ).compareTo( l.s );
			} else {
				return new Integer( l.e ).compareTo( e );
			}
		}

		public boolean equals( Lecture l ) {
			return compareTo( l ) == 0;
		}
	}

	final static int MAX_N = 100000;
	final static int MAX_K = 100;
	final static int MAX_E = 1000000000;

	void run() throws IOException {
		int[] t = parser.nextIntegersSeparatedBySpace( 2 );
		int n = t[0];
		int k = t[1];
		check( ( n >= 1 ) && ( n <= MAX_N ) && ( k >= 0 ) && ( k <= MAX_K ),
				"Bad value of n or k: " + n + " " + k );
		Lecture[] l1 = new Lecture[n + 1];
		for ( int i = 0; i < n; i ++ ) {
			t = parser.nextIntegersSeparatedBySpace( 2 );
			l1[i] = new Lecture( t[0], t[1] );
			check( ( 0 <= l1[i].s ) && ( l1[i].s <= l1[i].e ) && ( l1[i].e <= MAX_E ),
					"Bad times of " + i + "-th lecture ends at line " + ( i + 1 ) );
		}
		parser.checkEOF();
		l1[n] = new Lecture( -1, -1 );
		Arrays.sort( l1 );
		int m = 0;
		Lecture[] l = new Lecture[n + 1];
		l[0] = l1[0];
		for ( int i = 1; i <= n; i ++ ) {
			if ( l1[i].e > l[m].e && l1[i].e > l1[i].s ) {
				l[++ m] = l1[i];
			}
		}
		int[][][] dp = new int[k + 1][m + 1][2];
		for ( int i = 0; i < k; i ++ ) {
			int jump = 1;
			for ( int j = 0; j < m; j ++ ) {
				for ( ; jump <= m && l[jump].s < l[j].e; jump ++ ) {
				}
				dp[i][j + 1][0] = Math.max( dp[i][j + 1][0], dp[i][j][0] );
				dp[i + 1][j + 1][1] = Math.max( dp[i + 1][j + 1][1],
						dp[i][j][0] + l[j + 1].e - l[j + 1].s );
				dp[i + 1][jump - 1][1] = Math.max( dp[i + 1][jump - 1][1],
						dp[i][j][1] + l[jump - 1].e - Math.max( l[jump - 1].s, l[j].e ) );
				if ( jump > m ) {
					continue;
				}
				dp[i][jump][0] = Math.max( dp[i][jump][0], dp[i][j][1] );
				dp[i + 1][jump][1] = Math.max( dp[i + 1][jump][1], dp[i][j][1] + l[jump].e - l[jump].s );
			}
		}
		int r = 0;
		for ( int i = 0; i <= k; i ++ ) {
			for ( int j = 0; j <= m; j ++ ) {
				r = Math.max( r, Math.max( dp[i][j][0], dp[i][j][1] ) );
			}
		}
		out.println( r );
	}	

	void check( boolean f, String msg ) {
		if ( ! f ) {
			out.close();
			throw new RuntimeException( msg );
		}
	}

	class Parser {
		BufferedReader in;
		int line;

		Parser( BufferedReader in ) {
			this.in = in;
			line = 0;
		}

		private void checkL( boolean f, String msg ) {
			check( f, "Line " + line + ": " + msg );
		}

		void checkEOF() throws IOException {
			check( in.readLine() == null, "EOF expected" );
		}

		String nextLine() throws IOException {
			line ++;
		    String s = in.readLine();
		    checkL( s != null, "Unexpected end of file" );
		    return s;
		}

		int[] nextIntegersSeparatedBySpace( int k ) throws IOException {
			String[] s = nextLine().split( " " );
			checkL( s.length == k, "Wrong format" );
			int[] r = new int[k];
			for ( int i = 0; i < k; i ++ ) {
				try {
					r[i] = new Integer( s[i] );
				} catch ( Exception e ) {
					checkL( false, "Not a number at position " + ( i + 1 ) );
				}
			}
			return r;
		}
	}
	
	public static void main( String[] args ) throws IOException {
		new Thread() {

			public void run() {
				try {
					timetable_sb solution = new timetable_sb();
					solution.open();
					solution.run();
					solution.close();
				} catch ( Exception e ) {
					throw new RuntimeException( e );
				}
			}
		}.start();
	}
}
